==> micronaut-serde-support-1.5.2-sources.jar: io/micronaut/serde/support/serdes/SqlTimestampSerde.java <==
/*
 * Copyright 2017-2021 original authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.micronaut.serde.support.serdes;

import java.io.IOException;
import java.sql.Timestamp;
import java.time.Instant;

import io.micronaut.context.annotation.Secondary;
import io.micronaut.core.type.Argument;
import io.micronaut.serde.Decoder;
import io.micronaut.serde.Deserializer;
import io.micronaut.serde.Encoder;
import io.micronaut.serde.Serializer;
import io.micronaut.serde.exceptions.SerdeException;
import io.micronaut.serde.util.NullableSerde;
import jakarta.inject.Singleton;

/**
 * Serde for SQL timestamps.
 */
@Singleton
@Secondary
final class SqlTimestampSerde implements NullableSerde<Timestamp> {
    private static final Argument<Instant> INSTANT_ARGUMENT = Argument.of(Instant.class);
    private final InstantSerde instantSerde;

    SqlTimestampSerde(InstantSerde instantSerde) {
        this.instantSerde = instantSerde;
    }

    @Override
    public Serializer<Timestamp> createSpecific(EncoderContext encoderContext, Argument<? extends Timestamp> type) {
        final Argument<Instant> argument = Argument.of(Instant.class, type.getAnnotationMetadata());
        final Serializer<Instant> specific = instantSerde.createSpecific(
                encoderContext, argument
        );
        if (specific != instantSerde) {
            return (encoder, context, t, value) -> specific.serialize(
                    encoder,
                    context,
                    argument, value.toInstant()
            );
        }
        return this;
    }

    @Override
    public Deserializer<Timestamp> createSpecific(DecoderContext decoderContext, Argument<? super Timestamp> context)
            throws SerdeException {
        final Argument<Instant> argument = Argument.of(Instant.class, context.getAnnotationMetadata());
        final Deserializer<Instant> specific = instantSerde.createSpecific(decoderContext, argument);
        if (specific != instantSerde) {
            return (decoder, subContext, type) -> {
                final Instant i = specific.deserialize(
                        decoder,
                        subContext,
                        argument
                );
                if (i != null) {
                    return Timestamp.from(i);
                }
                return null;
            };
        }
        return this;
    }

    @Override
    public void serialize(Encoder encoder, EncoderContext context, Argument<? extends Timestamp> type, Timestamp value) throws IOException {
        instantSerde.serialize(encoder, context, INSTANT_ARGUMENT, value.toInstant());
    }

    @Override
    public Timestamp deserializeNonNull(Decoder decoder, DecoderContext decoderContext, Argument<? super Timestamp> type)
            throws IOException {
        return Timestamp.from(instantSerde.deserializeNonNull(
                decoder,
                decoderContext,
                INSTANT_ARGUMENT
        ));
    }
}

==> micronaut-serde-support-2.8.1-sources.jar: io/micronaut/serde/support/serdes/SqlTimestampSerde.java <==
/*
 * Copyright 2017-2021 original authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.micronaut.serde.support.serdes;

import io.micronaut.context.annotation.Secondary;
import io.micronaut.core.type.Argument;
import io.micronaut.serde.Decoder;
import io.micronaut.serde.Deserializer;
import io.micronaut.serde.Encoder;
import io.micronaut.serde.Serde;
import io.micronaut.serde.Serializer;
import io.micronaut.serde.exceptions.SerdeException;
import jakarta.inject.Singleton;

import java.io.IOException;
import java.sql.Timestamp;
import java.time.Instant;

/**
 * Serde for SQL timestamps.
 */
@Singleton
@Secondary
final class SqlTimestampSerde implements Serde<Timestamp> {
    private static final Argument<Instant> INSTANT_ARGUMENT = Argument.of(Instant.class);
    private final InstantSerde instantSerde;

    SqlTimestampSerde(InstantSerde instantSerde) {
        this.instantSerde = instantSerde;
    }

    @Override
    public Serializer<Timestamp> createSpecific(EncoderContext encoderContext, Argument<? extends Timestamp> type) {
        final Argument<Instant> argument = Argument.of(Instant.class, type.getAnnotationMetadata());
        final Serializer<Instant> specific = instantSerde.createSpecific(
                encoderContext, argument
        );
        if (specific != instantSerde) {
            return (encoder, context, t, value) -> specific.serialize(
                    encoder,
                    context,
                    argument, value.toInstant()
            );
        }
        return this;
    }

    @Override
    public Deserializer<Timestamp> createSpecific(DecoderContext decoderContext, Argument<? super Timestamp> context)
            throws SerdeException {
        final Argument<Instant> argument = Argument.of(Instant.class, context.getAnnotationMetadata());
        final Deserializer<Instant> specific = instantSerde.createSpecific(decoderContext, argument);
        if (specific != instantSerde) {
            return (decoder, subContext, type) -> {
                final Instant i = specific.deserialize(
                        decoder,
                        subContext,
                        argument
                );
                if (i != null) {
                    return Timestamp.from(i);
                }
                return null;
            };
        }
        return this;
    }

    @Override
    public void serialize(Encoder encoder, EncoderContext context, Argument<? extends Timestamp> type, Timestamp value) throws IOException {
        instantSerde.serialize(encoder, context, INSTANT_ARGUMENT, value.toInstant());
    }

    @Override
    public Timestamp deserialize(Decoder decoder, DecoderContext decoderContext, Argument<? super Timestamp> type) throws IOException {
        return Timestamp.from(instantSerde.deserialize(
                decoder,
                decoderContext,
                INSTANT_ARGUMENT
        ));
    }
}
